"use strict";
const command_1 = require("@oclif/command");
const execa = require("execa");
const fs = require("fs");
const Listr = require("listr");
const path = require("path");
class Enveigle extends command_1.Command {
    async run() {
        const { flags } = this.parse(Enveigle);
        const temporaryPlaybook = {
            name: 'playbook',
            src: path.join(__dirname, '../templates/enveigle.yml'),
            dest: 'enveigle.yml',
        };
        const temporaries = [
            temporaryPlaybook,
            {
                name: 'no_hosts_matched_callback',
                src: path.join(__dirname, '../templates/no_hosts_matched.py'),
                dest: flags.callback_dir + '/no_hosts_matcheded.py',
            },
        ];
        const tasks = new Listr([
            {
                title: 'Copy temporary files',
                task: () => temporaries.forEach(temporary => fs.copyFileSync(temporary.src, temporary.dest))
            },
            {
                title: 'Template .env files to local system',
                task: (ctx, task) => {
                    task.title = `$ ansible-playbook ${temporaryPlaybook.dest} -e env=${flags.env}`;
                    return execa('ansible-playbook', [temporaryPlaybook.dest, `-e env=${flags.env}`])
                        .catch(err => {
                        if (err.code === 10) {
                            task.skip('Could not match supplied host pattern. Try the `--env` flag!');
                            return;
                        }
                        ctx.ansibleErr = err;
                        task.skip('Something went wrong. Try again!');
                    });
                }
            },
            {
                title: 'Cleanup temporary files',
                task: () => temporaries.forEach(temporary => fs.unlinkSync(temporary.dest))
            },
            {
                title: 'Re-throw ansible error',
                enabled: ctx => ctx.ansibleErr,
                task: ctx => {
                    throw ctx.ansibleErr;
                }
            }
        ]);
        tasks.run().catch(err => {
            console.error('');
            console.error('##########################################');
            console.error('Abort! Something went wrong');
            console.error('Error message:');
            console.error(err);
        });
    }
}
Enveigle.description = 'Deceive Ansible to template Trellis .env files to local Bedrock';
Enveigle.flags = {
    // add --version flag to show CLI version
    version: command_1.flags.version({ char: 'v' }),
    help: command_1.flags.help({ char: 'h' }),
    env: command_1.flags.string({
        char: 'e',
        description: 'local environment name',
        env: 'ENVEIGLE_ENV',
        default: 'development',
    }),
    callback_dir: command_1.flags.string({
        char: 'c',
        description: 'ansible callback directory',
        env: 'ENVEIGLE_CALLBACK_DIR',
        default: 'lib/trellis/plugins/callback',
    }),
};
module.exports = Enveigle;
